package httpbearer

import (
	"testing"
)

func TestTrimLeft(t *testing.T) {

	tests := []struct{
		Value string
		Expected string
	}{
		{
			Value: "",
			Expected: "",
		},



		{
			Value: " ",
			Expected: "",
		},
		{
			Value: "  ",
			Expected: "",
		},
		{
			Value: "   ",
			Expected: "",
		},
		{
			Value: "    ",
			Expected: "",
		},
		{
			Value: "     ",
			Expected: "",
		},



		{
			Value: "\t",
			Expected: "",
		},
		{
			Value: "\t\t",
			Expected: "",
		},
		{
			Value: "\t\t\t",
			Expected: "",
		},
		{
			Value: "\t\t\t\t",
			Expected: "",
		},
		{
			Value: "\t\t\t\t\t",
			Expected: "",
		},



		{
			Value: "\r\n",
			Expected: "",
		},
		{
			Value: "\r\n\r\n",
			Expected: "",
		},
		{
			Value: "\r\n\r\n\r\n",
			Expected: "",
		},
		{
			Value: "\r\n\r\n\r\n\r\n",
			Expected: "",
		},
		{
			Value: "\r\n\r\n\r\n\r\n\r\n",
			Expected: "",
		},



		{
			Value: "\t \r\n \r\n\t \t\r\n   \r\n\r\n",
			Expected: "",
		},



		{
			Value:   " ToK3n]]*i4HD2@)",
			Expected: "ToK3n]]*i4HD2@)",
		},
		{
			Value:  "  ToK3n]]*i4HD2@)",
			Expected: "ToK3n]]*i4HD2@)",
		},
		{
			Value: "   ToK3n]]*i4HD2@)",
			Expected: "ToK3n]]*i4HD2@)",
		},
		{
			Value: "    ToK3n]]*i4HD2@)",
			Expected:  "ToK3n]]*i4HD2@)",
		},
		{
			Value: "     ToK3n]]*i4HD2@)",
			Expected:   "ToK3n]]*i4HD2@)",
		},



		{
			Value:  "\tToK3n]]*i4HD2@)",
			Expected: "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\t\tToK3n]]*i4HD2@)",
			Expected:  "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\t\t\tToK3n]]*i4HD2@)",
			Expected:    "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\t\t\t\tToK3n]]*i4HD2@)",
			Expected:      "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\t\t\t\t\tToK3n]]*i4HD2@)",
			Expected:        "ToK3n]]*i4HD2@)",
		},



		{
			Value: "\r\nToK3n]]*i4HD2@)",
			Expected:  "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\r\n\r\nToK3n]]*i4HD2@)",
			Expected:      "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\r\n\r\n\r\nToK3n]]*i4HD2@)",
			Expected:          "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\r\n\r\n\r\n\r\nToK3n]]*i4HD2@)",
			Expected:              "ToK3n]]*i4HD2@)",
		},
		{
			Value: "\r\n\r\n\r\n\r\n\r\nToK3n]]*i4HD2@)",
			Expected:                  "ToK3n]]*i4HD2@)",
		},



		{
			Value: "\t \r\n \r\n\t \t\r\n   \r\n\r\nToK3n]]*i4HD2@",
			Expected:                              "ToK3n]]*i4HD2@",
		},
	}

	for testNumber, test := range tests {

		actual := trimleft(test.Value)
		expected := test.Expected

		if expected != actual {
			t.Errorf("For test #%d, the actual left-trimmed value is not what was expected.", testNumber)
			t.Logf("EXPECTED: %q", expected)
			t.Logf("ACTUAL:   %q", actual)
			t.Logf("VALUE:    %q", test.Value)
			continue
		}
	}
}
